/*===============================================================================================
 ブロック名   : ドライバブロック
 機能説明     : I2Cに関するI/Fを提供するCソースコード。
              : 
 Revision     : $1.00
 Date         : $2015/02/23
 Author       : $S.Shinagawa
 COPYRIGHT (C) NOGUE-LAB.COM ALL RIGHTS RESERVED
================================================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
 
#include <linux/i2c-dev.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include "drv_public.h"

/*===============================================================================================
 定数宣言
================================================================================================*/
#define DEFAULT_SLAVE_ADDR			(0x30)		/*デフォルトのスレーブアドレス値				*/
#define ADDR_LEN_1BYTE				(1)			/*スレーブ端末の参照先アドレス値のサイズ 1byte	*/
#define ADDR_LEN_2BYTE				(2)			/*スレーブ端末の参照先アドレス値のサイズ 2byte	*/
#define I2C_BUFF_MAX_LEN			(0x102)		/*I2Cバッファ最大長								*/
/*===============================================================================================
 グローバル変数
================================================================================================*/
static int gI2cFd;							/*ファイルディスクリプタ変数格納先					*/
static unsigned char gI2cBuffer[I2C_BUFF_MAX_LEN];	/*I2Cバッファ								*/
/*===============================================================================================
名称		I2cのポートオープン
	
書式		int DRV_openI2cC(char *pI2cDev)
	
引数		pSerialDev	char *	IN	デバイス名文字列
	
戻り値		DRV_OK	正常終了
			DRV_NG	異常終了

機能説明	I2CT用のポートをオープンする。
			I2Cポートのオープンは，gI2cFdのファイルディスクリプタ変数に入れる。
	
特記事項	I2Cデバイスが1個のみを想定して作られたもののため，I2Cデバイスが複数ある場合は，
			gI2cFdの取り扱い等，複数取り扱えるように変更しなければならない

修正履歴	
================================================================================================*/
int DRV_openI2c(char *pI2cDev)
{
	int slaveAddr = DEFAULT_SLAVE_ADDR;
	/*I2CポートをRead/Write属性でオープン*/
	gI2cFd = open(pI2cDev, O_RDWR);
	if (gI2cFd < 0){
		printf("Faild to open i2c port. gI2cFd = %d\n",gI2cFd);
		return DRV_NG;
	}

	/* 通信先アドレスの設定 */
	if (ioctl(gI2cFd, I2C_SLAVE, slaveAddr) < 0){
		printf("Unable to get bus access to talk to slave\n");
		return DRV_NG;
	}

	return DRV_OK;
}

/*===============================================================================================
名称		I2Cのポートクローズ
	
書式		void DRV_closeI2c(void)
	
引数		なし
	
戻り値		なし

機能説明	I2Cのポート設定を基の内容に設定し，ポートの接続をクローズする。
	
特記事項	

修正履歴	
================================================================================================*/
void DRV_closeI2c(void)
{
	close(gI2cFd);
}

/*===============================================================================================
名称		I2Cの通信先スレーブアドレスの変更
	
書式		int DRV_setSlaveAddr(int slaveAddr)
	
引数		slaveAddr	int	IN	スレーブアドレス
	
戻り値		DRV_OK	正常終了
			DRV_NG	異常終了

機能説明	I2Cの通信先スレーブアドレスの変更。
	
特記事項	DRV_openI2cにてオープン処理が既に済んでいること。

修正履歴	
================================================================================================*/
int DRV_setSlaveAddr(int slaveAddr)
{
	/* 通信先アドレスの設定 */
	if (ioctl(gI2cFd, I2C_SLAVE, slaveAddr) < 0){
		printf("Unable to get bus access to talk to slave\n");
		return DRV_NG;
	}
	
	return DRV_OK;
}
/*===============================================================================================
名称		I2Cの通信先スレーブへのデータ書込み
	
書式		int DRV_writeI2c(unsigned int addr,unsigned char *pBuf,unsigned int length,int addrType)
	
引数		addr		unsigned int	IN	スレーブ端末の参照先アドレス
			pBuf		unsigned char * IN	書込みデータ
			length		unsigned int	IN	書込みデータ長
			addrType	int				IN	スレーブ端末の参照先アドレス値のサイズの種類
	
戻り値		DRV_OK	正常終了
			DRV_NG	異常終了

機能説明	I2Cの通信先スレーブ端末にデータを書込む。
	
特記事項	DRV_openI2cにてオープン処理が既に済んでいること。

修正履歴	
================================================================================================*/
int DRV_writeI2c(unsigned int addr,unsigned char *pBuf,unsigned int length,int addrType)
{
	int addrLen;
	int rlt;
	
	switch(addrType){
	case ADDR_TYPE_1BYTE:
		gI2cBuffer[0] = (unsigned char)(addr&0x000000ff);
		addrLen = ADDR_LEN_1BYTE;
		break;
	case ADDR_TYPE_2BYTE:
		gI2cBuffer[0] = (unsigned char)((addr&0x0000ff00) >> 8);
		gI2cBuffer[1] = (unsigned char)( addr&0x000000ff);
		addrLen = ADDR_LEN_2BYTE;
		break;
	default:
		printf("DRV_writeI2c function. address type error :%d\n",addrType);
		return DRV_NG;
	}

	if(I2C_BUFF_MAX_LEN < (addrLen+length)){
		printf("DRV_writeI2c function. send data length error :%d byte(MAX:%d byte)\n",length,(I2C_BUFF_MAX_LEN-addrLen));
		return DRV_NG;
	}
	
	memcpy(&gI2cBuffer[addrLen],pBuf,length);

	rlt = write(gI2cFd,gI2cBuffer,addrLen+length);
	if(rlt != (addrLen+length)){
		printf("DRV_writeI2c function. Error writting to i2c slave\n");
		return DRV_NG;
	}

	return DRV_OK;
}

/*===============================================================================================
名称		I2Cの通信先スレーブからのデータ読込み
	
書式		int DRV_readI2c(unsigned int addr,unsigned char *pBuf,unsigned int length,int addrType)
	
引数		addr		unsigned int	IN	スレーブ端末の参照先アドレス
			pBuf		unsigned char * OUT	読込みデータ
			length		unsigned int	IN	読込みデータ長
			addrType	int				IN	スレーブ端末の参照先アドレス値のサイズの種類
	
戻り値		DRV_OK	正常終了
			DRV_NG	異常終了

機能説明	I2Cの通信先スレーブ端末からデータを読込む。
	
特記事項	DRV_openI2cにてオープン処理が既に済んでいること。

修正履歴	
================================================================================================*/
int DRV_readI2c(unsigned int addr,unsigned char *pBuf,unsigned int length,int addrType)
{
	unsigned char setAddrBuffer[ADDR_LEN_2BYTE];
	int addrLen;
	int rlt;
	
	switch(addrType){
	case ADDR_TYPE_1BYTE:
		setAddrBuffer[0] = (unsigned char)(addr&0x000000ff);
		addrLen = ADDR_LEN_1BYTE;
		break;
	case ADDR_TYPE_2BYTE:
		setAddrBuffer[0] = (unsigned char)((addr&0x0000ff00) >> 8);
		setAddrBuffer[1] = (unsigned char)( addr&0x000000ff);
		addrLen = ADDR_LEN_2BYTE;
		break;
	default:
		printf("DRV_readI2c function. address type error :%d\n",addrType);
		return DRV_NG;
	}

	rlt = write(gI2cFd,setAddrBuffer,addrLen);
	if(rlt != addrLen){
		printf("DRV_readI2c function. Error writeing to i2c slave rlt = %d(0x%x)\n",rlt,rlt);
		return DRV_NG;
	}

	rlt = read(gI2cFd,pBuf,length);
	if(rlt !=length){
		printf("DRV_readI2c function. Error reading from i2c slave rlt = %d(0x%x)\n",rlt,rlt);
		return DRV_NG;
	}

	return DRV_OK;
}
/*=========================================== END ==============================================*/
